"""
Provides ovoenergy version information.
"""

# This file is auto-generated! Do not edit!
# Use `python -m incremental.update ovoenergy` to change this file.

from incremental import Version

__version__ = Version("ovoenergy", 2, 0, 1, dev=0)
__all__ = ["__version__"]
